package it.unimi.evotion.tasks;

import org.apache.phoenix.query.QueryServices;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.SaveMode;
import org.apache.spark.sql.SparkSession;
import org.apache.spark.sql.functions;

import java.security.InvalidParameterException;

import it.unimi.evotion.tasks.Task;


public class Csv2hbase implements Task {

    // static final String PHOENIX_DRIVER = "org.apache.phoenix.jdbc.PhoenixDriver";
    // static final String MARIADB_DRIVER = "org.mariadb.jdbc.Driver";
    static final String UNIMI_DRIVER = "it.unimi.jdbc.wrap.Driver";

    // static SparkSession spark;

    /**
     * Save a CSV in a HBase table
     *
     * @param args
     *      args[0] url_csv
     *      args[1] table_name
     *      args[2] primary_keys | AUTO
     *      args[2] url_hbase
     *
     * Note:
     *      url_csv
     *          path of the a folder, generated from a previous task, or a CSV file
     *
     *      table_name
     *          name of the table in the HBASE db to populate
     *
     *      primary_keys | AUTO
     *          each HBASE table MUST have a primary key.
     *          With "AUTO" a autoincremented column with name "ROW_ID" is added
     *
     *      url_database
     *          jdbc url to access the database
     */
    public static void main(String[] args) throws Exception {

        Csv2hbase task = new Csv2hbase();
        task.init(args);
        task.run();
        task.postProcessing();

        // spark = SparkSession
        //     .builder()
        //     // .master("local[2]")
        //     // .master("yarn")
        //     .appName("Csv2hbase")
        //     .getOrCreate();
        //
        // // check the number of parameters
        // if (args.length < 4)
        //     throw new InvalidParameterException("Missing parameters (url_csv, table_name, primary_key, url_hbase)");
        //
        // // extract the parameters
        // String urlcsv = args[0];
        // String tablename = args[1].replace("-", "");
        // String primaryKey = args[2];
        // String urldb  = args[3];
        //
        // // apply the patch to the Spark JDBC access
        // Class.forName(UNIMI_DRIVER);
        // urldb = urldb.replace("jdbc:", "jdbc:wrap:");
        //
        // // load the dataset on the file
        // Dataset<Row> df = spark.read()
        //     .format("csv")
        //     .option("header", "true")
        //     .option("inferSchema", "true")
        //     .load(urlcsv);
        //
        // if ("AUTO".equalsIgnoreCase(primaryKey)) {
        //     primaryKey = "ROW_ID";
        //     df = df.withColumn("ROW_ID", functions.monotonically_increasing_id());
        // }
        //
        // //  write the dataset on the dabase's table
        // df.write()
        //     .format("jdbc")
        //     .mode(SaveMode.Overwrite)
        //     .option("url", urldb)
        //     .option("driver", UNIMI_DRIVER)
        //     .option("dbtable", tablename)
        //     .option("primarykey", primaryKey)
        //     .option(QueryServices.MAX_MUTATION_SIZE_ATTRIB,"10000000")
        //     .option(QueryServices.IMMUTABLE_ROWS_ATTRIB,"10000000")
        //     .save();
        //
        // spark.stop();
        // spark.close();
    }

    private SparkSession spark;
    private String urlcsv;
    private String tablename;
    private String primaryKey;
    private String urldb;
    private Dataset<Row> df;

    public void init(Object...args) throws Exception {

        // check the number of parameters
        if (args.length < 4)
            throw new InvalidParameterException("Missing parameters (url_csv, table_name, primary_key, url_hbase)");

        // extract the parameters
        this.urlcsv = (String) args[0];
        this.tablename = ((String)args[1]).replace("-", "");
        this.primaryKey = (String) args[2];
        this.urldb  = (String) args[3];

        spark = SparkSession
            .builder()
            .master("local[2]")
            // .master("yarn")
            .appName("Csv2hbase")
            .getOrCreate();

        Class.forName(UNIMI_DRIVER);
        this.urldb = urldb.replace("jdbc:", "jdbc:wrap:");
    }

    public void run(Object...params) {

        // load the dataset on the file
        this.df = spark.read()
            .format("csv")
            .option("header", "true")
            .option("inferSchema", "true")
            .load(urlcsv);

        if ("AUTO".equalsIgnoreCase(primaryKey)) {
            primaryKey = "ROW_ID";
            df = df.withColumn("ROW_ID", functions.monotonically_increasing_id());
        }

    }

    public void postProcessing(Object...params) {

        //  write the dataset on the dabase's table
        this.df.write()
            .format("jdbc")
            .mode(SaveMode.Overwrite)
            .option("url", urldb)
            .option("driver", UNIMI_DRIVER)
            .option("dbtable", tablename)
            .option("primarykey", primaryKey)
            .option(QueryServices.MAX_MUTATION_SIZE_ATTRIB,"10000000")
            .option(QueryServices.IMMUTABLE_ROWS_ATTRIB,"10000000")
            .save();

        this.spark.stop();
        this.spark.close();
    }

}
