package it.unimi.evotion.tasks;

import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.SaveMode;
import org.apache.spark.sql.SparkSession;

import java.security.InvalidParameterException;

import it.unimi.evotion.tasks.Task;

public class Hbase2csv implements Task {

    // static final String PHOENIX_DRIVER = "org.apache.phoenix.jdbc.PhoenixDriver";
    // static final String MARIADB_DRIVER = "org.mariadb.jdbc.Driver";
    static final String UNIMI_DRIVER = "it.unimi.jdbc.wrap.Driver";

    static SparkSession spark;

    /**
     * Save a CSV in a HBase table
     *
     * @param args
     *      args[0] url_csv
     *      args[1] table_name
     *      args[2] primary_keys | AUTO
     *      args[2] url_hbase
     *
     * Note:
     *      url_csv
     *          path of the a folder, generated from a previous task, or a CSV file
     *
     *      table_name
     *          name of the table in the HBASE db to populate
     *
     *      primary_keys | AUTO
     *          each HBASE table MUST have a primary key.
     *          With "AUTO" a autoincremented column with name "ROW_ID" is added
     *
     *      url_database
     *          jdbc url to access the database
     */
    public static void main(String[] args) throws Exception {

        Hbase2csv task = new Hbase2csv();
        task.init(args);
        task.run();
        task.postProcessing();

        // spark = SparkSession
        //     .builder()
        //     // .master("yarn")
        //     // .master("local[2]")
        //     .appName("Hbase2csv")
        //     .getOrCreate();
        //
        // // check the number of parameters
        // if (args.length < 3)
        //     throw new InvalidParameterException("Missing parameters (url_hbase, table_name, url_csv)");
        //
        // // extract the parameters
        // String urldb  = args[0];
        // String tablename = args[1];     // table or SELECT ....
        // String urlcsv = args[2];
        //
        // // apply the patch to the Spark JDBC access
        // Class.forName(UNIMI_DRIVER);
        // urldb = urldb.replace("jdbc:", "jdbc:wrap:");
        //
        // // load the dataset from the dabase's table
        // Dataset<Row> df;
        //
        // if (tablename.toUpperCase().startsWith("SELECT"))
        //     tablename = String.format("(%s) AS TMP", tablename);
        //
        // df = spark.read()
        //     .format("jdbc")
        //     .option("url", urldb)
        //     .option("driver", UNIMI_DRIVER)
        //     .option("dbtable", tablename)
        //     .load();
        //
        // // write the dataset on the file
        // String urltmp = urlcsv + ".tmp";
        // urltmp = urlcsv;
        //
        // df.write()
        //     .format("csv")
        //     .mode(SaveMode.Overwrite)
        //     .option("header", "true")
        //     .save(urltmp);
        //
        // spark.stop();
        // spark.close();
    }

    private String urldb;
    private String tablename;
    private String urlcsv;
    private Dataset<Row> df;

    public void init(Object...args) throws Exception {

        // check the number of parameters
        if (args.length < 3)
            throw new InvalidParameterException("Missing parameters (url_hbase, table_name, url_csv)");

        // extract the parameters
        this.urldb  = (String) args[0];
        this.tablename = (String) args[1];     // table or SELECT ....
        this.urlcsv = (String) args[2];

        // apply the patch to the Spark JDBC access
        Class.forName(UNIMI_DRIVER);
        this.urldb = urldb.replace("jdbc:", "jdbc:wrap:");

        if (this.tablename.toUpperCase().startsWith("SELECT"))
            this.tablename = String.format("(%s) AS TMP", this.tablename);
    }

    public void run(Object...params) throws Exception {

        df = spark.read()
            .format("jdbc")
            .option("url", urldb)
            .option("driver", UNIMI_DRIVER)
            .option("dbtable", tablename)
            .load();
    }

    public void postProcessing(Object...params) throws Exception {

        // write the dataset on the file
        String urltmp = urlcsv + ".tmp";
        urltmp = urlcsv;

        df.write()
            .format("csv")
            .mode(SaveMode.Overwrite)
            .option("header", "true")
            .save(urltmp);

        spark.stop();
        spark.close();
    }

}
